"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TrackedChangeMethod = exports.Group = void 0;
const internal_1 = require("../../internal");
class Group extends internal_1.EnhancedState {
    constructor(collection, initialItems = [], config = {}) {
        super(collection.agileInstance(), initialItems, config);
        this._output = [];
        this.nextGroupOutput = [];
        this._preciseItemKeys = [];
        this.observers = {};
        this.notFoundItemKeys = [];
        this.loadedInitialValue = true;
        this.observers['value'] = new internal_1.StateObserver(this, {
            key: config.key,
        });
        this.observers['output'] = new internal_1.GroupObserver(this, {
            key: config.key,
        });
        this.collection = () => collection;
        this.addSideEffect(Group.rebuildGroupSideEffectKey, (state, config) => {
            var _a;
            this.rebuild(((_a = config === null || config === void 0 ? void 0 : config.any) === null || _a === void 0 ? void 0 : _a.trackedChanges) || [], config);
        });
        this.rebuild();
    }
    get output() {
        internal_1.ComputedTracker.tracked(this.observers['output']);
        return internal_1.copy(this._output);
    }
    set output(value) {
        internal_1.LogCodeManager.log('1C:03:00', [this._key]);
    }
    has(itemKey) {
        return this.value.indexOf(itemKey) !== -1;
    }
    get size() {
        return this.value.length;
    }
    remove(itemKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const notExistingItemKeysInCollection = [];
        const notExistingItemKeys = [];
        let newGroupValue = internal_1.copy(this.nextStateValue);
        const updatedPreciseItemKeys = internal_1.copy(this._preciseItemKeys);
        config = internal_1.defineConfig(config, {
            softRebuild: true,
            any: {},
        });
        config.any['trackedChanges'] = [];
        _itemKeys.forEach((itemKey) => {
            const exists = newGroupValue.includes(itemKey);
            if (!exists) {
                notExistingItemKeys.push(itemKey);
                notExistingItemKeysInCollection.push(itemKey);
                return;
            }
            if (config.softRebuild) {
                const index = updatedPreciseItemKeys.findIndex((ik) => ik === itemKey);
                if (index !== -1) {
                    updatedPreciseItemKeys.splice(index, 1);
                    config.any['trackedChanges'].push({
                        index,
                        method: TrackedChangeMethod.REMOVE,
                        key: itemKey,
                    });
                }
            }
            if (!this.collection().getItem(itemKey))
                notExistingItemKeysInCollection.push(itemKey);
            newGroupValue = newGroupValue.filter((key) => key !== itemKey);
        });
        if (notExistingItemKeys.length >= _itemKeys.length)
            return this;
        if (notExistingItemKeysInCollection.length >= _itemKeys.length)
            config.background = true;
        this.set(newGroupValue, internal_1.removeProperties(config, ['softRebuild']));
        return this;
    }
    add(itemKeys, config = {}) {
        const _itemKeys = internal_1.normalizeArray(itemKeys);
        const notExistingItemKeysInCollection = [];
        const existingItemKeys = [];
        const newGroupValue = internal_1.copy(this.nextStateValue);
        const updatedPreciseItemKeys = internal_1.copy(this._preciseItemKeys);
        config = internal_1.defineConfig(config, {
            method: 'push',
            softRebuild: true,
            any: {},
        });
        config.any['trackedChanges'] = [];
        _itemKeys.forEach((itemKey) => {
            const exists = newGroupValue.includes(itemKey);
            if (!this.collection().getItem(itemKey))
                notExistingItemKeysInCollection.push(itemKey);
            if (exists) {
                existingItemKeys.push(itemKey);
                return;
            }
            if (config.softRebuild) {
                const index = config.method === 'push' ? updatedPreciseItemKeys.length : 0;
                updatedPreciseItemKeys.push(itemKey);
                config.any['trackedChanges'].push({
                    index,
                    method: TrackedChangeMethod.ADD,
                    key: itemKey,
                });
            }
            newGroupValue[config.method || 'push'](itemKey);
        });
        if (existingItemKeys.length >= _itemKeys.length)
            return this;
        if (notExistingItemKeysInCollection.concat(existingItemKeys).length >=
            _itemKeys.length)
            config.background = true;
        this.set(newGroupValue, internal_1.removeProperties(config, ['method', 'softRebuild']));
        return this;
    }
    replace(oldItemKey, newItemKey, config = {}) {
        const newGroupValue = internal_1.copy(this._value);
        newGroupValue.splice(newGroupValue.indexOf(oldItemKey), 1, newItemKey);
        this.set(newGroupValue, config);
        return this;
    }
    getItems() {
        return this.value
            .map((itemKey) => this.collection().getItem(itemKey))
            .filter((item) => item !== undefined);
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = this._key;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            loadValue: true,
            followCollectionPersistKeyPattern: true,
            storageKeys: [],
            defaultStorageKey: null,
        });
        if (_config.followCollectionPersistKeyPattern) {
            key = internal_1.CollectionPersistent.getGroupStorageKey(key || this._key, this.collection()._key);
        }
        super.persist(key, {
            loadValue: _config.loadValue,
            storageKeys: _config.storageKeys,
            defaultStorageKey: _config.defaultStorageKey,
        });
        return this;
    }
    rebuild(trackedChanges = [], config = {}) {
        if (!this.collection().isInstantiated)
            return this;
        const notFoundItemKeys = [];
        if (trackedChanges.length > 0) {
            trackedChanges.forEach((change) => {
                const item = this.collection().getItem(change.key);
                switch (change.method) {
                    case TrackedChangeMethod.ADD:
                        if (item != null) {
                            this._preciseItemKeys.splice(change.index, 0, change.key);
                            this.nextGroupOutput.splice(change.index, 0, internal_1.copy(item._value));
                        }
                        else {
                            notFoundItemKeys.push(change.key);
                        }
                        break;
                    case TrackedChangeMethod.UPDATE:
                        if (item != null) {
                            this.nextGroupOutput[change.index] = internal_1.copy(item._value);
                        }
                        else {
                            notFoundItemKeys.push(change.key);
                        }
                        break;
                    case TrackedChangeMethod.REMOVE:
                        this._preciseItemKeys.splice(change.index, 1);
                        this.nextGroupOutput.splice(change.index, 1);
                        break;
                    default:
                        break;
                }
            });
            this.observers['output'].ingest(config);
        }
        else {
            const groupItemValues = [];
            this._preciseItemKeys = [];
            this._value.forEach((itemKey) => {
                const item = this.collection().getItem(itemKey);
                if (item != null) {
                    groupItemValues.push(item._value);
                    this._preciseItemKeys.push(itemKey);
                }
                else
                    notFoundItemKeys.push(itemKey);
            });
            this.observers['output'].ingestOutput(groupItemValues, config);
        }
        if (notFoundItemKeys.length > 0 && this.loadedInitialValue) {
            internal_1.LogCodeManager.log('1C:02:00', [this.collection()._key, this._key], notFoundItemKeys);
        }
        this.notFoundItemKeys = notFoundItemKeys;
        return this;
    }
}
exports.Group = Group;
Group.rebuildGroupSideEffectKey = 'rebuildGroup';
var TrackedChangeMethod;
(function (TrackedChangeMethod) {
    TrackedChangeMethod[TrackedChangeMethod["ADD"] = 0] = "ADD";
    TrackedChangeMethod[TrackedChangeMethod["REMOVE"] = 1] = "REMOVE";
    TrackedChangeMethod[TrackedChangeMethod["UPDATE"] = 2] = "UPDATE";
})(TrackedChangeMethod = exports.TrackedChangeMethod || (exports.TrackedChangeMethod = {}));
