var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { State, ComputedTracker, removeProperties, LogCodeManager, isAsyncFunction, extractRelevantObservers, defineConfig, } from '../internal';
export class Computed extends State {
    constructor(agileInstance, computeFunction, config = {}) {
        super(agileInstance, null, {
            key: config.key,
            dependents: config.dependents,
        });
        this.deps = new Set();
        this.hardCodedDeps = [];
        this.isComputed = true;
        config = defineConfig(config, {
            computedDeps: [],
            autodetect: !isAsyncFunction(computeFunction),
        });
        this.agileInstance = () => agileInstance;
        this.computeFunction = computeFunction;
        this.config = {
            autodetect: config.autodetect,
        };
        this.hardCodedDeps = extractRelevantObservers(config.computedDeps).filter((dep) => dep !== undefined);
        this.deps = new Set(this.hardCodedDeps);
        this.deps.forEach((observer) => {
            observer.addDependent(this.observers['value']);
        });
        this.recompute({ autodetect: config.autodetect, overwrite: true });
    }
    recompute(config = {}) {
        config = defineConfig(config, {
            autodetect: false,
        });
        this.compute({ autodetect: config.autodetect }).then((result) => {
            this.observers['value'].ingestValue(result, removeProperties(config, ['autodetect']));
        });
        return this;
    }
    updateComputeFunction(computeFunction, deps = [], config = {}) {
        config = defineConfig(config, {
            autodetect: this.config.autodetect,
        });
        this.deps.forEach((observer) => {
            observer.removeDependent(this.observers['value']);
        });
        this.hardCodedDeps = extractRelevantObservers(deps).filter((dep) => dep !== undefined);
        this.deps = new Set(this.hardCodedDeps);
        this.deps.forEach((observer) => {
            observer.addDependent(this.observers['value']);
        });
        this.computeFunction = computeFunction;
        this.recompute(removeProperties(config, ['overwriteDeps']));
        return this;
    }
    compute(config = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            config = defineConfig(config, {
                autodetect: this.config.autodetect,
            });
            if (config.autodetect)
                ComputedTracker.track();
            const computedValue = this.computeFunction();
            if (config.autodetect) {
                const foundDeps = ComputedTracker.getTrackedObservers();
                this.deps.forEach((observer) => {
                    if (!foundDeps.includes(observer) &&
                        !this.hardCodedDeps.includes(observer)) {
                        this.deps.delete(observer);
                        observer.removeDependent(this.observers['value']);
                    }
                });
                foundDeps.forEach((observer) => {
                    if (!this.deps.has(observer)) {
                        this.deps.add(observer);
                        observer.addDependent(this.observers['value']);
                    }
                });
            }
            return computedValue;
        });
    }
    persist() {
        LogCodeManager.log('19:03:00');
        return this;
    }
}
