import { Observer, copy, equal, notEqual, isFunction, StateRuntimeJob, createArrayFromObject, generateId, defineConfig, removeProperties, } from '../internal';
export class StateObserver extends Observer {
    constructor(state, config = {}) {
        super(state.agileInstance(), defineConfig(config, {
            value: state._value,
        }));
        this.state = () => state;
        this.nextStateValue = copy(state._value);
    }
    ingest(config = {}) {
        const state = this.state();
        if (state.isComputed) {
            state.compute().then((result) => {
                this.ingestValue(result, config);
            });
        }
        else {
            this.ingestValue(state.nextStateValue, config);
        }
    }
    ingestValue(newStateValue, config = {}) {
        var _a;
        const state = this.state();
        config = defineConfig(config, {
            perform: true,
            force: false,
        });
        if (state.isPlaceholder) {
            config.force = true;
            config.overwrite = true;
        }
        this.nextStateValue = state.computeValueMethod
            ? copy(state.computeValueMethod(newStateValue))
            : copy(newStateValue);
        if (equal(state._value, this.nextStateValue) && !config.force)
            return;
        const job = new StateRuntimeJob(this, Object.assign(Object.assign({}, removeProperties(config, ['perform'])), {
            key: (_a = config.key) !== null && _a !== void 0 ? _a : `${this._key != null ? this._key + '_' : ''}${generateId()}_value`,
        }));
        this.agileInstance().runtime.ingest(job, {
            perform: config.perform,
        });
    }
    perform(job) {
        const observer = job.observer;
        const state = observer.state();
        state.previousStateValue = copy(state._value);
        state._value = copy(observer.nextStateValue);
        state.nextStateValue = copy(observer.nextStateValue);
        if (job.config.overwrite) {
            state.initialStateValue = copy(state._value);
            state.previousStateValue = copy(state._value);
            state.isPlaceholder = false;
        }
        state.isSet = notEqual(state._value, state.initialStateValue);
        this.sideEffects(job);
        job.observer.previousValue = copy(observer.value);
        job.observer.value = copy(state._value);
    }
    sideEffects(job) {
        var _a, _b, _c;
        const state = job.observer.state();
        if ((_b = (_a = job.config) === null || _a === void 0 ? void 0 : _a.sideEffects) === null || _b === void 0 ? void 0 : _b.enabled) {
            const sideEffectArray = createArrayFromObject(state.sideEffects);
            sideEffectArray.sort(function (a, b) {
                return b.instance.weight - a.instance.weight;
            });
            for (const sideEffect of sideEffectArray) {
                if (isFunction(sideEffect.instance.callback)) {
                    if (!((_c = job.config.sideEffects.exclude) === null || _c === void 0 ? void 0 : _c.includes(sideEffect.key)))
                        sideEffect.instance.callback(job.observer.state(), job.config);
                }
            }
        }
    }
}
