"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.State = void 0;
const internal_1 = require("../internal");
class State {
    constructor(agileInstance, initialValue, config = {}) {
        this.isSet = false;
        this.isPlaceholder = false;
        this.observers = {};
        this.sideEffects = {};
        config = internal_1.defineConfig(config, {
            dependents: [],
            isPlaceholder: false,
        });
        this.agileInstance = () => agileInstance;
        this._key = config.key;
        this.observers['value'] = new internal_1.StateObserver(this, {
            key: config.key,
            dependents: config.dependents,
        });
        this.initialStateValue = internal_1.copy(initialValue);
        this._value = internal_1.copy(initialValue);
        this.previousStateValue = internal_1.copy(initialValue);
        this.nextStateValue = internal_1.copy(initialValue);
        this.isPlaceholder = true;
        if (!config.isPlaceholder)
            this.set(initialValue, { overwrite: true });
    }
    set value(value) {
        this.set(value);
    }
    get value() {
        internal_1.ComputedTracker.tracked(this.observers['value']);
        return internal_1.copy(this._value);
    }
    set key(value) {
        this.setKey(value);
    }
    get key() {
        return this._key;
    }
    setKey(value) {
        this._key = value;
        for (const observerKey in this.observers)
            this.observers[observerKey]._key = value;
        return this;
    }
    set(value, config = {}) {
        config = internal_1.defineConfig(config, {
            force: false,
        });
        const _value = internal_1.isFunction(value)
            ? value(internal_1.copy(this._value))
            : value;
        this.observers['value'].ingestValue(_value, config);
        return this;
    }
    ingest(config = {}) {
        this.observers['value'].ingest(config);
        return this;
    }
    addSideEffect(key, callback, config = {}) {
        config = internal_1.defineConfig(config, {
            weight: 10,
        });
        if (!internal_1.isFunction(callback)) {
            internal_1.LogCodeManager.log('00:03:01', ['Side Effect Callback', 'function']);
            return this;
        }
        this.sideEffects[key] = {
            callback: callback,
            weight: config.weight,
        };
        return this;
    }
    removeSideEffect(key) {
        delete this.sideEffects[key];
        return this;
    }
    hasSideEffect(key) {
        return !!this.sideEffects[key];
    }
}
exports.State = State;
