import { CreateStorageConfigInterface, Storage, Storages, CreateStoragesConfigInterface, CreateAgileSubInstanceInterface } from '../internal';
export * from './storages';
/**
 * Returns a newly created Storage.
 *
 * A Storage Class serves as an interface to external storages,
 * such as the [Async Storage](https://github.com/react-native-async-storage/async-storage) or
 * [Local Storage](https://www.w3schools.com/html/html5_webstorage.asp).
 *
 * It creates the foundation to easily [`persist()`](https://agile-ts.org/docs/core/state/methods#persist) [Agile Sub Instances](https://agile-ts.org/docs/introduction/#agile-sub-instance)
 * (like States or Collections) in nearly any external storage.
 *
 * [Learn more..](https://agile-ts.org/docs/core/agile-instance/methods#createstorage)
 *
 * @public
 * @param config - Configuration object
 */
export declare function createStorage(config: CreateStorageConfigInterface): Storage;
/**
 * Returns a newly created Storage Manager.
 *
 * A Storage Manager manages all external Storages for AgileTs
 * and provides an interface to easily store,
 * load and remove values from multiple external Storages at once.
 *
 * @param config - Configuration object
 */
export declare function createStorageManager(config?: CreateStorageManagerConfigInterfaceWithAgile): Storages;
/**
 * Returns the shared Storage Manager
 * or creates a new one when no shared Storage Manager exists.
 */
export declare function getStorageManager(): Storages;
/**
 * Assigns the specified Storage Manager
 * as default (shared) Storage Manager for all Agile Instances.
 *
 *  @param instance - Storage Manager to be registered as the default Storage Manager.
 */
export declare const assignSharedAgileStorageManager: (instance: Storages | null) => void;
export interface CreateStorageManagerConfigInterfaceWithAgile extends CreateAgileSubInstanceInterface, CreateStoragesConfigInterface {
}
